summaryrefslogtreecommitdiff
path: root/src/lib
diff options
context:
space:
mode:
authorCarl Hetherington <cth@carlh.net>2015-05-13 23:44:54 +0100
committerCarl Hetherington <cth@carlh.net>2015-05-13 23:44:54 +0100
commit65274489101d5b5c503ea94d61d7ac5eb1372636 (patch)
tree2d11e71e33f78ef3224af436c8c93525b4135cfc /src/lib
parentedecc975384bfef50418e3fb77e5c4629207ec08 (diff)
parent419c025709132e21ce948c4a6051a323f06ee61d (diff)
Merge branch '2.0' of git.carlh.net:git/dcpomatic into 2.0
Diffstat (limited to 'src/lib')
-rw-r--r--src/lib/audio_analysis.cc96
-rw-r--r--src/lib/audio_analysis.h25
-rw-r--r--src/lib/content.cc1
-rw-r--r--src/lib/ffmpeg_decoder.cc53
-rw-r--r--src/lib/ffmpeg_decoder.h2
-rw-r--r--src/lib/job.cc1
-rw-r--r--src/lib/job_manager.cc1
-rw-r--r--src/lib/server_finder.cc1
-rw-r--r--src/lib/signal_manager.cc (renamed from src/lib/ui_signaller.cc)9
-rw-r--r--src/lib/signal_manager.h (renamed from src/lib/ui_signaller.h)16
-rw-r--r--src/lib/signaller.h6
-rw-r--r--src/lib/update.cc1
-rw-r--r--src/lib/wscript2
13 files changed, 90 insertions, 124 deletions
diff --git a/src/lib/audio_analysis.cc b/src/lib/audio_analysis.cc
index ee34b0d80..73422a9be 100644
--- a/src/lib/audio_analysis.cc
+++ b/src/lib/audio_analysis.cc
@@ -1,5 +1,5 @@
/*
- Copyright (C) 2012 Carl Hetherington <cth@carlh.net>
+ Copyright (C) 2012-2015 Carl Hetherington <cth@carlh.net>
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
@@ -18,15 +18,17 @@
*/
#include "audio_analysis.h"
-#include "dcpomatic_assert.h"
#include "cross.h"
+#include "util.h"
+#include "raw_convert.h"
+#include <libxml++/libxml++.h>
#include <boost/filesystem.hpp>
+#include <boost/foreach.hpp>
#include <stdint.h>
-#include <inttypes.h>
#include <cmath>
-#include <cassert>
#include <cstdio>
#include <iostream>
+#include <inttypes.h>
using std::ostream;
using std::istream;
@@ -35,6 +37,7 @@ using std::vector;
using std::cout;
using std::max;
using std::list;
+using boost::shared_ptr;
AudioPoint::AudioPoint ()
{
@@ -43,14 +46,10 @@ AudioPoint::AudioPoint ()
}
}
-AudioPoint::AudioPoint (FILE* f)
+AudioPoint::AudioPoint (cxml::ConstNodePtr node)
{
- for (int i = 0; i < COUNT; ++i) {
- int n = fscanf (f, "%f", &_data[i]);
- if (n != 1) {
- _data[i] = 0;
- }
- }
+ _data[PEAK] = node->number_child<float> ("Peak");
+ _data[RMS] = node->number_child<float> ("RMS");
}
AudioPoint::AudioPoint (AudioPoint const & other)
@@ -75,14 +74,12 @@ AudioPoint::operator= (AudioPoint const & other)
}
void
-AudioPoint::write (FILE* f) const
+AudioPoint::as_xml (xmlpp::Element* parent) const
{
- for (int i = 0; i < COUNT; ++i) {
- fprintf (f, "%f\n", _data[i]);
- }
+ parent->add_child ("Peak")->add_child_text (raw_convert<string> (_data[PEAK]));
+ parent->add_child ("RMS")->add_child_text (raw_convert<string> (_data[RMS]));
}
-
AudioAnalysis::AudioAnalysis (int channels)
{
_data.resize (channels);
@@ -90,44 +87,21 @@ AudioAnalysis::AudioAnalysis (int channels)
AudioAnalysis::AudioAnalysis (boost::filesystem::path filename)
{
- FILE* f = fopen_boost (filename, "r");
- if (!f) {
- throw OpenFileError (filename);
- }
+ cxml::Document f ("AudioAnalysis");
+ f.read_file (filename);
- int channels = 0;
- fscanf (f, "%d", &channels);
- _data.resize (channels);
+ BOOST_FOREACH (cxml::NodePtr i, f.node_children ("Channel")) {
+ vector<AudioPoint> channel;
- for (int i = 0; i < channels; ++i) {
- int points;
- fscanf (f, "%d", &points);
- if (feof (f)) {
- fclose (f);
- return;
- }
-
- for (int j = 0; j < points; ++j) {
- _data[i].push_back (AudioPoint (f));
- if (feof (f)) {
- fclose (f);
- return;
- }
+ BOOST_FOREACH (cxml::NodePtr j, i->node_children ("Point")) {
+ channel.push_back (AudioPoint (j));
}
- }
- /* These may not exist in old analysis files, so be careful
- about reading them.
- */
-
- float peak;
- DCPTime::Type peak_time;
- if (fscanf (f, "%f%" SCNd64, &peak, &peak_time) == 2) {
- _peak = peak;
- _peak_time = DCPTime (peak_time);
+ _data.push_back (channel);
}
-
- fclose (f);
+
+ _peak = f.number_child<float> ("Peak");
+ _peak_time = DCPTime (f.number_child<DCPTime::Type> ("PeakTime"));
}
void
@@ -160,26 +134,20 @@ AudioAnalysis::points (int c) const
void
AudioAnalysis::write (boost::filesystem::path filename)
{
- boost::filesystem::path tmp = filename;
- tmp.replace_extension (".tmp");
-
- FILE* f = fopen_boost (tmp, "w");
- if (!f) {
- throw OpenFileError (tmp);
- }
+ shared_ptr<xmlpp::Document> doc (new xmlpp::Document);
+ xmlpp::Element* root = doc->create_root_node ("AudioAnalysis");
- fprintf (f, "%ld\n", _data.size ());
- for (vector<vector<AudioPoint> >::iterator i = _data.begin(); i != _data.end(); ++i) {
- fprintf (f, "%ld\n", i->size ());
- for (vector<AudioPoint>::iterator j = i->begin(); j != i->end(); ++j) {
- j->write (f);
+ BOOST_FOREACH (vector<AudioPoint>& i, _data) {
+ xmlpp::Element* channel = root->add_child ("Channel");
+ BOOST_FOREACH (AudioPoint& j, i) {
+ j.as_xml (channel->add_child ("Point"));
}
}
if (_peak) {
- fprintf (f, "%f%" PRId64, _peak.get (), _peak_time.get().get ());
+ root->add_child("Peak")->add_child_text (raw_convert<string> (_peak.get ()));
+ root->add_child("PeakTime")->add_child_text (raw_convert<string> (_peak_time.get().get ()));
}
- fclose (f);
- boost::filesystem::rename (tmp, filename);
+ doc->write_to_file_formatted (filename.string ());
}
diff --git a/src/lib/audio_analysis.h b/src/lib/audio_analysis.h
index 1872c57ad..9387ec896 100644
--- a/src/lib/audio_analysis.h
+++ b/src/lib/audio_analysis.h
@@ -17,21 +17,16 @@
*/
-/** @file src/lib/audio_analysis.h
- * @brief AudioAnalysis and AudioPoint classes.
- */
-
#ifndef DCPOMATIC_AUDIO_ANALYSIS_H
#define DCPOMATIC_AUDIO_ANALYSIS_H
-#include "types.h"
+#include <vector>
+#include <list>
#include <boost/filesystem.hpp>
#include <boost/optional.hpp>
-#include <vector>
+#include <libcxml/cxml.h>
+#include "types.h"
-/** @class AudioPoint
- * @brief A single point of an audio analysis for one portion of one channel.
- */
class AudioPoint
{
public:
@@ -42,11 +37,11 @@ public:
};
AudioPoint ();
- AudioPoint (FILE *);
+ AudioPoint (cxml::ConstNodePtr node);
AudioPoint (AudioPoint const &);
AudioPoint& operator= (AudioPoint const &);
- void write (FILE *) const;
+ void as_xml (xmlpp::Element *) const;
float& operator[] (int t) {
return _data[t];
@@ -56,14 +51,6 @@ private:
float _data[COUNT];
};
-/** @class AudioAnalysis
- * @brief An analysis of the audio data in a piece of AudioContent.
- *
- * This is a set of AudioPoints for each channel. The AudioPoints
- * each represent some measurement of the audio over a portion of the
- * content. For example each AudioPoint may give the RMS level of
- * a 1-minute portion of the audio.
- */
class AudioAnalysis : public boost::noncopyable
{
public:
diff --git a/src/lib/content.cc b/src/lib/content.cc
index b00ea3e57..b9e8367e1 100644
--- a/src/lib/content.cc
+++ b/src/lib/content.cc
@@ -24,7 +24,6 @@
#include "content.h"
#include "util.h"
#include "content_factory.h"
-#include "ui_signaller.h"
#include "exceptions.h"
#include "film.h"
#include "safe_stringstream.h"
diff --git a/src/lib/ffmpeg_decoder.cc b/src/lib/ffmpeg_decoder.cc
index 6122a547e..35e15a331 100644
--- a/src/lib/ffmpeg_decoder.cc
+++ b/src/lib/ffmpeg_decoder.cc
@@ -453,15 +453,43 @@ FFmpegDecoder::decode_subtitle_packet ()
AVSubtitleRect const * rect = sub.rects[0];
- if (rect->type != SUBTITLE_BITMAP) {
- throw DecodeError (_("non-bitmap subtitles not yet supported"));
+ switch (rect->type) {
+ case SUBTITLE_NONE:
+ break;
+ case SUBTITLE_BITMAP:
+ decode_bitmap_subtitle (rect, period);
+ break;
+ case SUBTITLE_TEXT:
+ cout << "XXX: SUBTITLE_TEXT " << rect->text << "\n";
+ break;
+ case SUBTITLE_ASS:
+ cout << "XXX: SUBTITLE_ASS " << rect->ass << "\n";
+ break;
}
+
+ avsubtitle_free (&sub);
+}
+
+list<ContentTimePeriod>
+FFmpegDecoder::image_subtitles_during (ContentTimePeriod p, bool starting) const
+{
+ return _ffmpeg_content->subtitles_during (p, starting);
+}
+list<ContentTimePeriod>
+FFmpegDecoder::text_subtitles_during (ContentTimePeriod, bool) const
+{
+ return list<ContentTimePeriod> ();
+}
+
+void
+FFmpegDecoder::decode_bitmap_subtitle (AVSubtitleRect const * rect, ContentTimePeriod period)
+{
/* Note RGBA is expressed little-endian, so the first byte in the word is R, second
G, third B, fourth A.
*/
shared_ptr<Image> image (new Image (PIX_FMT_RGBA, dcp::Size (rect->w, rect->h), true));
-
+
/* Start of the first line in the subtitle */
uint8_t* sub_p = rect->pict.data[0];
/* sub_p looks up into a BGRA palette which is here
@@ -470,7 +498,7 @@ FFmpegDecoder::decode_subtitle_packet ()
uint32_t const * palette = (uint32_t *) rect->pict.data[1];
/* Start of the output data */
uint32_t* out_p = (uint32_t *) image->data()[0];
-
+
for (int y = 0; y < rect->h; ++y) {
uint8_t* sub_line_p = sub_p;
uint32_t* out_line_p = out_p;
@@ -481,7 +509,7 @@ FFmpegDecoder::decode_subtitle_packet ()
sub_p += rect->pict.linesize[0];
out_p += image->stride()[0] / sizeof (uint32_t);
}
-
+
dcp::Size const vs = _ffmpeg_content->video_size ();
dcpomatic::Rect<double> const scaled_rect (
static_cast<double> (rect->x) / vs.width,
@@ -489,20 +517,7 @@ FFmpegDecoder::decode_subtitle_packet ()
static_cast<double> (rect->w) / vs.width,
static_cast<double> (rect->h) / vs.height
);
-
- image_subtitle (ContentTimePeriod (period.from, period.to), image, scaled_rect);
- avsubtitle_free (&sub);
-}
-
-list<ContentTimePeriod>
-FFmpegDecoder::image_subtitles_during (ContentTimePeriod p, bool starting) const
-{
- return _ffmpeg_content->subtitles_during (p, starting);
+ image_subtitle (period, image, scaled_rect);
}
-list<ContentTimePeriod>
-FFmpegDecoder::text_subtitles_during (ContentTimePeriod, bool) const
-{
- return list<ContentTimePeriod> ();
-}
diff --git a/src/lib/ffmpeg_decoder.h b/src/lib/ffmpeg_decoder.h
index 616fa88dd..6f027ce1c 100644
--- a/src/lib/ffmpeg_decoder.h
+++ b/src/lib/ffmpeg_decoder.h
@@ -64,6 +64,8 @@ private:
void decode_audio_packet ();
void decode_subtitle_packet ();
+ void decode_bitmap_subtitle (AVSubtitleRect const * rect, ContentTimePeriod period);
+
void maybe_add_subtitle ();
boost::shared_ptr<AudioBuffers> deinterleave_audio (uint8_t** data, int size);
diff --git a/src/lib/job.cc b/src/lib/job.cc
index f28146632..c4d93ddc1 100644
--- a/src/lib/job.cc
+++ b/src/lib/job.cc
@@ -27,7 +27,6 @@
#include "job.h"
#include "util.h"
#include "cross.h"
-#include "ui_signaller.h"
#include "exceptions.h"
#include "film.h"
#include "log.h"
diff --git a/src/lib/job_manager.cc b/src/lib/job_manager.cc
index 63db662d0..b5b64a77e 100644
--- a/src/lib/job_manager.cc
+++ b/src/lib/job_manager.cc
@@ -26,7 +26,6 @@
#include "job_manager.h"
#include "job.h"
#include "cross.h"
-#include "ui_signaller.h"
using std::string;
using std::list;
diff --git a/src/lib/server_finder.cc b/src/lib/server_finder.cc
index 72a9a4ef5..b4b400b52 100644
--- a/src/lib/server_finder.cc
+++ b/src/lib/server_finder.cc
@@ -22,7 +22,6 @@
#include "util.h"
#include "config.h"
#include "cross.h"
-#include "ui_signaller.h"
#include "dcpomatic_socket.h"
#include "raw_convert.h"
#include <libcxml/cxml.h>
diff --git a/src/lib/ui_signaller.cc b/src/lib/signal_manager.cc
index 4cb34da51..7c2b3e11a 100644
--- a/src/lib/ui_signaller.cc
+++ b/src/lib/signal_manager.cc
@@ -1,5 +1,5 @@
/*
- Copyright (C) 2012 Carl Hetherington <cth@carlh.net>
+ Copyright (C) 2012-2015 Carl Hetherington <cth@carlh.net>
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
@@ -17,8 +17,7 @@
*/
-#include "ui_signaller.h"
-
-/** Global UISignaller instance */
-UISignaller* ui_signaller = 0;
+#include "signal_manager.h"
+/** Global SignalManager instance */
+SignalManager* signal_manager = 0;
diff --git a/src/lib/signaller.h b/src/lib/signaller.h
index 408cfcf5b..4ef9b38b3 100644
--- a/src/lib/signaller.h
+++ b/src/lib/signaller.h
@@ -20,7 +20,7 @@
#ifndef DCPOMATIC_SIGNALLER_H
#define DCPOMATIC_SIGNALLER_H
-#include "ui_signaller.h"
+#include "signal_manager.h"
#include <boost/thread/mutex.hpp>
#include <boost/signals2.hpp>
@@ -100,8 +100,8 @@ public:
void emit (T signal)
{
Wrapper<T>* w = new Wrapper<T> (signal);
- if (ui_signaller) {
- ui_signaller->emit (boost::bind (&Wrapper<T>::signal, w));
+ if (signal_manager) {
+ signal_manager->emit (boost::bind (&Wrapper<T>::signal, w));
}
boost::mutex::scoped_lock lm (_mutex);
diff --git a/src/lib/update.cc b/src/lib/update.cc
index 26944ecc3..f433ff991 100644
--- a/src/lib/update.cc
+++ b/src/lib/update.cc
@@ -19,7 +19,6 @@
#include "update.h"
#include "version.h"
-#include "ui_signaller.h"
#include "safe_stringstream.h"
#include "config.h"
#include "util.h"
diff --git a/src/lib/wscript b/src/lib/wscript
index 68897a4c4..5956c73d6 100644
--- a/src/lib/wscript
+++ b/src/lib/wscript
@@ -109,7 +109,7 @@ sources = """
transcode_job.cc
transcoder.cc
types.cc
- ui_signaller.cc
+ signal_manager.cc
update.cc
upmixer_a.cc
util.cc
diff --git a/src/lib/ui_signaller.h b/src/lib/signal_manager.h
index 9d4495cd1..ae4306e30 100644
--- a/src/lib/ui_signaller.h
+++ b/src/lib/signal_manager.h
@@ -1,5 +1,5 @@
/*
- Copyright (C) 2012 Carl Hetherington <cth@carlh.net>
+ Copyright (C) 2012-2015 Carl Hetherington <cth@carlh.net>
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
*/
-#ifndef DCPOMATIC_UI_SIGNALLER_H
-#define DCPOMATIC_UI_SIGNALLER_H
+#ifndef DCPOMATIC_SIGNAL_MANAGER_H
+#define DCPOMATIC_SIGNAL_MANAGER_H
#include <boost/bind.hpp>
#include <boost/asio.hpp>
@@ -29,11 +29,11 @@ class Signaller;
/** A class to allow signals to be emitted from non-UI threads and handled
* by a UI thread.
*/
-class UISignaller : public boost::noncopyable
+class SignalManager : public boost::noncopyable
{
public:
- /** Create a UISignaller. Must be called from the UI thread */
- UISignaller ()
+ /** Create a SignalManager. Must be called from the UI thread */
+ SignalManager ()
: _work (_service)
{
_ui_thread = boost::this_thread::get_id ();
@@ -53,7 +53,7 @@ public:
/** This should wake the UI and make it call ui_idle() */
virtual void wake_ui () {
- /* This is only a sensible implementation when there is no GUI... */
+ /* This is only a sensible implementation when there is no GUI */
ui_idle ();
}
@@ -85,6 +85,6 @@ private:
boost::thread::id _ui_thread;
};
-extern UISignaller* ui_signaller;
+extern SignalManager* signal_manager;
#endif